@@ -35,7 +35,7 @@
 #if !defined(HAVE_STRCASECMP)
 #   define strcasecmp(a,b) strcmp(a,b)
 #endif
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || defined(__MINGW32__)
 #   define SIZE_T_FMTSTR "Iu"
 #else
 #   define SIZE_T_FMTSTR "zu"
@@ -60,6 +60,7 @@ static const char *helpstr[] = {
 "",
 "OPTIONS",
 "-d,--data         Print data with header information",
+"-f,--format whos  Turn on 'whos' display mode",
 "-h,--human        Human readable sizes in 'whos' display mode",
 "-v,--verbose      Turn on verbose messages",
 "-H,--help         This output",
@@ -72,11 +73,11 @@ static const char *helpstr[] = {
 NULL
 };
 
-static char *mxclass[15] = { "mxCELL_CLASS", "mxSTRUCT_CLASS", "mxOBJECT_CLASS",
+static const char *mxclass[15] = { "mxCELL_CLASS", "mxSTRUCT_CLASS", "mxOBJECT_CLASS",
                              "mxCHAR_CLASS", "mxSPARSE_CLASS", "mxDOUBLE_CLASS",
                              "mxSINGLE_CLASS", "mxINT8_CLASS", "mxUINT8_CLASS",
                              "mxINT16_CLASS", "mxUINT16_CLASS", "mxINT32_CLASS",
-                             "mxUINT32_CLASS","mxINT64_CLASS","mxUINT64_CLASS",
+                             "mxUINT32_CLASS","mxINT64_CLASS","mxUINT64_CLASS"
                             };
 static int printdata = 0;
 static int human_readable = 0;
@@ -91,8 +92,8 @@ static void (*printfunc)(matvar_t *matvar) = NULL;
 static char *
 get_next_token(char *str)
 {
-    char *tok, *tokens = "(){}.";
-    char *next_tok;
+    const char *tokens = "(){}.";
+    char *next_tok, *tok;
 
     next_tok = NULL;
     while (*tokens != '\0') {
@@ -183,8 +184,6 @@ slab_get_select(char *open, char *close, int rank, int *start, int *stride,
                     start[dim] = i-1;
                 else if ( nvals == 1 )
                     stride[dim] = i;
-                else if ( nvals == 1 )
-                    edge[dim] = i;
                 else if ( nvals == 2 )
                     edge[dim] = i;
                 else
@@ -312,11 +311,12 @@ slab_select_valid(int rank,int *start,int *stride,int *edge,matvar_t *matvar)
 }
 
 static void
-read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
+read_selected_data(mat_t *mat,matvar_t **_matvar,char *index_str)
 {
     char *next_tok_pos, next_tok = 0;
     char *open = NULL, *close = NULL;
     int err, i = 0, j, done = 0;
+    matvar_t* matvar = *_matvar;
 
     next_tok_pos = get_next_token(index_str);
     next_tok = *next_tok_pos;
@@ -345,9 +345,9 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
             }
             /* Get the rank of the dataset */
             rank   = slab_get_rank(open,close);
-            start  = malloc(rank*sizeof(int));
-            stride = malloc(rank*sizeof(int));
-            edge   = malloc(rank*sizeof(int));
+            start  = (int*)malloc(rank*sizeof(int));
+            stride = (int*)malloc(rank*sizeof(int));
+            edge   = (int*)malloc(rank*sizeof(int));
             for ( j = 0; j < rank; j++ ) {
                 start[j]  = 0;
                 stride[j] = 1;
@@ -363,7 +363,7 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
                 if ( matvar->isComplex ) {
                     mat_complex_split_t *z;
                     matvar->data = malloc(sizeof(*z));
-                    z = matvar->data;
+                    z = (mat_complex_split_t*)matvar->data;
                     z->Re = malloc(matvar->nbytes);
                     z->Im = malloc(matvar->nbytes);
                 } else {
@@ -397,8 +397,8 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
             matvar_t *field;
             char *varname;
 
+            varname = next_tok_pos+1;
             if ( matvar->class_type == MAT_C_STRUCT ) {
-                varname = next_tok_pos+1;
                 next_tok_pos = get_next_token(next_tok_pos+1);
                 if ( next_tok_pos != varname ) {
                     next_tok = *next_tok_pos;
@@ -416,13 +416,15 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
                 field = Mat_VarDuplicate(field,1);
                 Mat_VarFree(matvar);
                 matvar = field;
+                if ( done == 1 ) {
+                    Mat_VarReadDataAll(mat, matvar);
+                }
             } else if ( matvar->class_type == MAT_C_CELL ) {
                 int ncells;
                 matvar_t *cell, **cells;
 
                 ncells = matvar->nbytes / matvar->data_size;
-                cells = matvar->data;
-                varname = next_tok_pos+1;
+                cells = (matvar_t**)matvar->data;
                 next_tok_pos = get_next_token(next_tok_pos+1);
                 if ( next_tok_pos != varname ) {
                     next_tok = *next_tok_pos;
@@ -474,9 +476,9 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
             }
             /* Get the rank of the dataset */
             rank   = slab_get_rank(open,close);
-            start  = malloc(rank*sizeof(int));
-            stride = malloc(rank*sizeof(int));
-            edge   = malloc(rank*sizeof(int));
+            start  = (int*)malloc(rank*sizeof(int));
+            stride = (int*)malloc(rank*sizeof(int));
+            edge   = (int*)malloc(rank*sizeof(int));
             for ( j = 0; j < rank; j++ ) {
                 start[j]  = 0;
                 stride[j] = 1;
@@ -491,12 +493,12 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
                     cells = Mat_VarGetCellsLinear(matvar,*start,
                                   *stride,*edge);
                     if (matvar->rank == 2 && matvar->dims[0] == 1) {
-                       matvar->dims[1] = *edge;
+                        matvar->dims[1] = *edge;
                     } else if (matvar->rank == 2 && matvar->dims[1] == 1) {
-                       matvar->dims[0] = *edge;
+                        matvar->dims[0] = *edge;
                     } else {
-                       matvar->rank = 1;
-                       matvar->dims[0] = *edge;
+                        matvar->rank = 1;
+                        matvar->dims[0] = *edge;
                     }
                 } else {
                     cells = Mat_VarGetCells(matvar,start,stride,edge);
@@ -525,13 +527,14 @@ read_selected_data(mat_t *mat,matvar_t *matvar,char *index_str)
                 break;
         }
     }
+    *_matvar = matvar;
 }
 
 static void
 print_whos(matvar_t *matvar)
 {
     int i;
-    size_t nbytes = 0;
+    size_t nbytes;
     char size[32] = {'\0',};
 
     if ( print_whos_first ) {
@@ -542,17 +545,15 @@ print_whos(matvar_t *matvar)
     if ( matvar->rank > 0 ) {
         int cnt = 0;
         printf("%8" SIZE_T_FMTSTR, matvar->dims[0]);
-        nbytes = matvar->dims[0];
         for ( i = 1; i < matvar->rank; i++ ) {
-            if ( ceil(log10(matvar->dims[i]))+1 < 32 )
+            if ( ceil(log10((double)matvar->dims[i]))+1 < 32 )
                 cnt += sprintf(size+cnt,"x%" SIZE_T_FMTSTR, matvar->dims[i]);
-            nbytes *= matvar->dims[i];
         }
         printf("%-10s",size);
-        nbytes *= Mat_SizeOfClass(matvar->class_type);
     } else {
         printf("                    ");
     }
+    nbytes = Mat_VarGetSize(matvar);
     if ( human_readable ) {
         if ( nbytes > 1073741824L )
             printf(" %10.1fG",(double)nbytes/1073741824.0);
@@ -561,9 +562,9 @@ print_whos(matvar_t *matvar)
         else if ( nbytes > 1024 )
             printf(" %10.1fK",(double)nbytes/1024.0);
         else
-            printf(" %10luB",nbytes);
+            printf(" %10" SIZE_T_FMTSTR "B",nbytes);
     } else {
-        printf("  %10lu",nbytes);
+        printf("  %10" SIZE_T_FMTSTR,nbytes);
     }
     printf("  %-18s\n",mxclass[matvar->class_type-1]);
 
@@ -594,12 +595,24 @@ print_default_number(enum matio_types type, void *data)
             break;
 #ifdef _mat_int64_t
         case MAT_T_INT64:
-            printf("%lld",*(mat_int64_t*)data);
+#if MATIO_HAVE_INTTYPES_H
+            printf("%" PRIi64,*(mat_int64_t*)data);
+#elif defined(_MSC_VER) && _MSC_VER >= 1200
+            printf("%I64i",*(mat_int64_t*)data);
+#else
+            printf("%lld",(long long)(*(mat_int64_t*)data));
+#endif
             break;
 #endif
 #ifdef _mat_uint64_t
         case MAT_T_UINT64:
-            printf("%llu",*(mat_uint64_t*)data);
+#if MATIO_HAVE_INTTYPES_H
+            printf("%" PRIu64,*(mat_uint64_t*)data);
+#elif defined(_MSC_VER) && _MSC_VER >= 1200
+            printf("%I64u",*(mat_uint64_t*)data);
+#else
+            printf("%llu",(unsigned long long)(*(mat_uint64_t*)data));
+#endif
             break;
 #endif
         case MAT_T_INT32:
@@ -635,9 +648,9 @@ print_default_numeric_2d(matvar_t *matvar)
 
     stride = Mat_SizeOf(matvar->data_type);
     if ( matvar->isComplex ) {
-        mat_complex_split_t *complex_data = matvar->data;
-        char *rp = complex_data->Re;
-        char *ip = complex_data->Im;
+        mat_complex_split_t *complex_data = (mat_complex_split_t*)matvar->data;
+        char *rp = (char*)complex_data->Re;
+        char *ip = (char*)complex_data->Im;
         for ( i = 0; i < matvar->dims[0]; i++ ) {
             for ( j = 0; j < matvar->dims[1]; j++ ) {
                 size_t idx = matvar->dims[0]*j+i;
@@ -649,7 +662,7 @@ print_default_numeric_2d(matvar_t *matvar)
             printf("\n");
         }
     } else {
-        char *data = matvar->data;
+        char *data = (char*)matvar->data;
         for ( i = 0; i < matvar->dims[0]; i++ ) {
             for ( j = 0; j < matvar->dims[1]; j++ ) {
                 size_t idx = matvar->dims[0]*j+i;
@@ -668,9 +681,9 @@ print_default_numeric_3d(matvar_t *matvar)
     size_t i, j, k, l, stride;
     stride = Mat_SizeOf(matvar->data_type);
     if ( matvar->isComplex ) {
-        mat_complex_split_t *complex_data = matvar->data;
-        char *rp = complex_data->Re;
-        char *ip = complex_data->Im;
+        mat_complex_split_t *complex_data = (mat_complex_split_t*)matvar->data;
+        char *rp = (char*)complex_data->Re;
+        char *ip = (char*)complex_data->Im;
         for ( k = 0; k < matvar->dims[2]; k++ ) {
             Mat_Message("%s(:,:,%lu) = ",matvar->name,k);
             indent++;
@@ -690,7 +703,7 @@ print_default_numeric_3d(matvar_t *matvar)
             printf("\n");
         }
     } else {
-        char *data = matvar->data;
+        char *data = (char*)matvar->data;
         for ( k = 0; k < matvar->dims[2]; k++ ) {
             Mat_Message("%s(:,:,%lu) = ",matvar->name,k);
             indent++;
@@ -805,7 +818,7 @@ print_default(matvar_t *matvar)
 int
 main (int argc, char *argv[])
 {
-    char *prog_name = "matdump";
+    const char *prog_name = "matdump";
     int   i, c, err = EXIT_SUCCESS;
     mat_t    *mat;
     matvar_t *matvar;
@@ -831,12 +844,17 @@ main (int argc, char *argv[])
                 Mat_VerbMessage(1,"Printing data\n");
                 break;
             case 'f':
-                if ( NULL != optarg && !strcmp(optarg,"whos") ) {
-                    printfunc = print_whos;
-                    break;
-                }
-                Mat_Warning("%s is not a recognized output format. "
+                if ( NULL != optarg ) {
+                    if ( !strcmp(optarg,"whos") ) {
+                        printfunc = print_whos;
+                    } else {
+                        Mat_Warning("%s is not a recognized output format. "
                               "Using default\n", optarg);
+                    }
+                } else {
+                    Mat_Warning("Missing output format. "
+                            "Using default\n", optarg);
+                }
                 break;
             case 'h':
                 human_readable = 1;
@@ -858,12 +876,14 @@ main (int argc, char *argv[])
         }
     }
 
-    if ( (argc-optind) < 1 )
-        Mat_Error("Must specify at least one argument");
+    if ( (argc-optind) < 1 ) {
+        Mat_Critical("Must specify at least one argument");
+        return EXIT_FAILURE;
+    }
 
     mat = Mat_Open( argv[optind],MAT_ACC_RDONLY );
     if ( NULL == mat ) {
-        Mat_Error("Error opening %s\n", argv[optind]);
+        Mat_Critical("Error opening %s\n", argv[optind]);
         return EXIT_FAILURE;
     }
 
@@ -888,7 +908,7 @@ main (int argc, char *argv[])
                         Mat_VarReadDataAll(mat,matvar);
                     } else {
                         *next_tok_pos = next_tok;
-                        read_selected_data(mat,matvar,next_tok_pos);
+                        read_selected_data(mat,&matvar,next_tok_pos);
                     }
                 }
                 (*printfunc)(matvar);
